@@ -29,6 +29,9 @@ import requests
 from concurrent.futures import ThreadPoolExecutor
 import functools
 import base64
+import threading
+import time
+from antgo.utils.utils import *
 try:
     import queue
 except:
@@ -36,9 +39,16 @@ except:
 
 
 class BaseHandler(tornado.web.RequestHandler):
-  waiting_queue = {}
   executor = ThreadPoolExecutor(10)
 
+  @property
+  def response_dict(self):
+    return self.settings.get('response_dict')
+
+  @property
+  def response_cond(self):
+    return self.settings.get('response_cond')
+
   @property
   def demo_name(self):
     return self.settings.get('demo_name', '-')
@@ -99,27 +109,47 @@ class BaseHandler(tornado.web.RequestHandler):
   def request_waiting_time(self):
     return self.settings['request_waiting_time']
 
-  def _transfer(self, data_key, data_value, data_type, demo_response):
+  def _transfer(self, data_key, data_value, data_type, demo_response, is_api_call):
+    file_uuid = str(uuid.uuid4())
     if data_type == 'IMAGE':
       if os.path.exists(data_value):
         data = os.path.normpath(data_value)
-        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output'))
-        demo_response['DATA'].update({data_key: {'DATA': '/static/output/%s' % data.split('/')[-1], 'TYPE': 'IMAGE'}})
+        file_ext = data.split('/')[-1].split('.')[-1]
+        if not is_api_call:
+          shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output', f'{file_uuid}.{file_ext}'))
+          demo_response['DATA'].update(
+            {data_key: {'DATA': '/static/output/%s' % f'{file_uuid}.{file_ext}', 'TYPE': 'IMAGE'}}
+          )
+        else:
+          with open(data, 'rb') as fp:
+            image_content = base64.b64encode(fp.read()).decode('utf-8')
+            demo_response['DATA'].update(
+              {data_key: {'DATA': image_content, 'TYPE': 'IMAGE'}}
+            )
     elif data_type == 'VIDEO':
       if os.path.exists(data_value):
         data = os.path.normpath(data_value)
-        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output'))
-        demo_response['DATA'].update({data_key: {'DATA': '/static/output/%s' % data.split('/')[-1], 'TYPE': 'VIDEO'}})
+        file_ext = data.split('/')[-1].split('.')[-1]
+        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output', f'{file_uuid}.{file_ext}'))
+        demo_response['DATA'].update(
+          {data_key: {'DATA': '/static/output/%s' % f'{file_uuid}.{file_ext}', 'TYPE': 'VIDEO'}}
+        )
     elif data_type == 'AUDIO':
       if os.path.exists(data_value):
         data = os.path.normpath(data_value)
-        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output'))
-        demo_response['DATA'].update({data_key: {'DATA': '/static/output/%s' % data.split('/')[-1], 'TYPE': 'AUDIO'}})
+        file_ext = data.split('/')[-1].split('.')[-1]
+        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output', f'{file_uuid}.{file_ext}'))
+        demo_response['DATA'].update(
+          {data_key: {'DATA': '/static/output/%s' % f'{file_uuid}.{file_ext}', 'TYPE': 'AUDIO'}}
+        )
     elif data_type == 'FILE':
       if os.path.exists(data_value):
         data = os.path.normpath(data_value)
-        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output'))
-        demo_response['DATA'].update({data_key: {'DATA': '/static/output/%s' % data.split('/')[-1], 'TYPE': 'FILE'}})
+        file_ext = data.split('/')[-1].split('.')[-1]
+        shutil.copy(data, os.path.join(self.demo_dump, 'static', 'output', f'{file_uuid}.{file_ext}'))
+        demo_response['DATA'].update(
+          {data_key: {'DATA': '/static/output/%s' % f'{file_uuid}.{file_ext}', 'TYPE': 'FILE'}}
+        )
     else:
       # string
       demo_response['DATA'].update({data_key: {'DATA': str(data_value), 'TYPE': 'STRING'}})
@@ -127,41 +157,50 @@ class BaseHandler(tornado.web.RequestHandler):
     return demo_response
 
   @run_on_executor
-  def waitingResponse(self, data_id):
-    if data_id in BaseHandler.waiting_queue:
-      # 阻塞等待响应
-      try:
-        response_data = BaseHandler.waiting_queue[data_id].get(timeout=self.request_waiting_time)
-      except:
-        # 空队列，直接返回空
-        BaseHandler.waiting_queue.pop(data_id)
-        return {}
+  def waitingResponse(self, data_id, time_stamp, is_api_call=False):
+    self.response_cond.acquire()
+    data_response = None
+    while True:
+      for k, v in self.response_dict.items():
+        if k == data_id:
+          data_response = v
+          break
 
-      # 清空等待队列s
-      BaseHandler.waiting_queue.pop(data_id)
+      if data_response is not None:
+        break
+      else:
+        current_time = time.time()
+        if current_time > time_stamp + self.request_waiting_time:
+          break
 
-      if response_data is None:
-        return {}
+        if not self.response_cond.wait(30):
+          # 等待10s，还无法获得锁，返回无效数据
+          break
 
-      processed_data = response_data['data']
-      if len(processed_data) == 0:
-        return {}
+    # 执行到这里，存在两种情况，（1）已经获得结果；（2）已经超时，但没有获得结果
+    if data_id in self.response_dict:
+      self.response_dict.pop(data_id)
+    self.response_cond.release()
 
-      # build output folder (static/output)
-      if not os.path.exists(os.path.join(self.demo_dump, 'static', 'output')):
-        os.makedirs(os.path.join(self.demo_dump, 'static', 'output'))
+    if data_response is None:
+      return {}
 
-      demo_response = {'DATA': {}}
-      demo_response = {'DATA': {'RESULT': {'DATA':'', 'TYPE': 'STRING'}}}
-      for data in processed_data[0]:
-        item_type = data['type']
-        item_data = data['data']
-        item_title = data['title']
-        
-        demo_response = self._transfer(item_title, item_data, item_type, demo_response)
-      return demo_response
+    processed_data = data_response['data']
+    if len(processed_data) == 0:
+      return {}
 
-    return {}
+    # build output folder (static/output)
+    if not os.path.exists(os.path.join(self.demo_dump, 'static', 'output')):
+      os.makedirs(os.path.join(self.demo_dump, 'static', 'output'))
+
+    demo_response = {'DATA': {'RESULT': '', 'TYPE': 'STRING'}}
+    for data in processed_data[0]:
+      item_type = data['type']
+      item_data = data['data']
+      item_title = data['title']
+
+      demo_response = self._transfer(item_title, item_data, item_type, demo_response, is_api_call)
+    return demo_response
 
   @run_on_executor
   def asynProcess(self, preprocess_type, data):
@@ -225,7 +264,7 @@ class BaseHandler(tornado.web.RequestHandler):
         if 'file_size' in self.demo_constraint:
           max_file_size = self.demo_constraint['file_size']
           fsize = os.path.getsize(file_path) / float(1024 * 1024)
-          if round(fsize,2) > max_file_size:
+          if round(fsize, 2) > max_file_size:
             return {'status': 400,
                       'code': 'InvalidImageSize', 
                       'message': 'The input file size is too large (>%f MB)'%float(max_file_size)}
@@ -276,24 +315,38 @@ class BaseHandler(tornado.web.RequestHandler):
     elif preprocess_type == 'API_QUERY':
       # 1.step base64解码
       # format {'image': '', 'video': None, 'params': [{'data': ,'type': , 'name': ,},{}]}
-      image_str = None
-      if 'image' in data:
-        image_str = data['image']
+      data_list = data
+      if type(data_list) == dict:
+        data_list = [data_list]
 
-      if image_str is None:
-        return {'status': 400,
-          'code': 'InvalidData', 
-          'message': 'Missing query data'}
+      multi_images = []
+      for data in data_list:
+        image_str = None
+        if 'image' in data:
+          image_str = data['image']
 
-      image_b = base64.b64decode(image_str)
+        if image_str is None or image_str == '':
+          return {
+            'status': 400,
+            'code': 'InvalidData',
+            'message': 'Missing query data'}
+
+        image_b = base64.b64decode(image_str)
+        multi_images.append(image_b)
+
+      if len(multi_images) == 1:
+        multi_images = multi_images[0]
+      params = data.get('params', {})
       return {
         'status': 200,
-        'data': {'image': image_b, 'params': data['params']}
+        'data': {'image': multi_images, 'params': params}
       }
 
+
 class IndexHandler(BaseHandler):
   def get(self):
     image_history_data = []
+
     history_folder = os.path.join(self.demo_dump, 'static', 'input')
     if os.path.exists(history_folder):
       for f in os.listdir(history_folder):
@@ -344,7 +397,7 @@ class ClientQueryHandler(BaseHandler):
 
     # 1.step parse query data
     model_datas = []
-    model_data_names = []
+    model_names = []
     model_data_types = []
     request_param = None
 
@@ -370,11 +423,11 @@ class ClientQueryHandler(BaseHandler):
           self.write(json.dumps(result))
           return
         
-        model_data, model_data_name, model_data_type = result['data']
+        model_data, model_name, model_data_type = result['data']
 
         # 保存
         model_datas.append(model_data)
-        model_data_names.append(model_data_name)
+        model_names.append(model_name)
         model_data_types.append(model_data_type)
     else:
       if len(self.request.files) == 0:
@@ -415,9 +468,9 @@ class ClientQueryHandler(BaseHandler):
         self.write(json.dumps(result))
         return
       
-      model_data, model_data_name, model_data_type = result['data']
+      model_data, model_name, model_data_type = result['data']
       model_datas.append(model_data)
-      model_data_names.append(model_data_name)
+      model_names.append(model_name)
       model_data_types.append(model_data_type)
 
       request_param = self.get_argument('params', None)
@@ -426,27 +479,24 @@ class ClientQueryHandler(BaseHandler):
 
     if request_param is None:
       request_param = {}
-      
+
     request_param.update({
-      'id': model_data_names[0]
+      'id': model_names[0]
     })
     
     # push to backend
     model_input = (model_datas[0], model_data_types[0], request_param)
 
-    # 设置等待队列
-    BaseHandler.waiting_queue[model_data_names[0]] = queue.Queue()
-
     # 发送到处理队列
     self.dataset_queue.put(model_input)
     
     # 异步等待响应(设置等待队列，并在异步线程中等待)
-    data_response = yield self.waitingResponse(model_data_names[0])
+    data_response = yield self.waitingResponse(model_names[0], time.time())
 
     # 绑定输入数据,返回
     data_response['INPUT_TYPE'] = model_data_types[0]
     if model_data_types[0] in ['FILE', 'IMAGE', 'AUDIO', 'VIDEO']:
-      data_response['INPUT'] = '/static/input/%s' % model_data_names[0]
+      data_response['INPUT'] = f'/static/input/{model_names[0]}'
     else:
       data_response['INPUT'] = model_datas[0]
 
@@ -485,22 +535,51 @@ class ClientCliQueryHandler(BaseHandler):
     request_param.update(result['data']['params'])
 
     # push to backend
-    model_input = (image, 'IMAGE_MEMORY', request_param)
-
-    # 设置等待队列
-    BaseHandler.waiting_queue[data_id] = queue.Queue()
+    data_type = 'IMAGE_MEMORY'
+    if type(image) == list:
+      data_type = 'IMAGE_MULTI_MEMORY'
+    model_input = (image, data_type, request_param)
 
     # 发送到处理队列
     self.dataset_queue.put(model_input)
     
     # 异步等待响应(设置等待队列，并在异步线程中等待)
-    data_response = yield self.waitingResponse(data_id)
+    data_response = yield self.waitingResponse(data_id, time.time(), True)
 
     # 返回成功
     self.write(json.dumps(data_response))
     self.finish()
 
 
+class ClientDownloadHandler(BaseHandler):
+  @gen.coroutine
+  def get(self):
+    file_name = self.get_argument('filename', None)
+    if file_name is None:
+      return
+
+    download_path = os.path.join(self.settings.get('static_path'), 'output', file_name)
+    if not os.path.exists(download_path):
+      return
+
+    # Content-Type
+    self.set_header('Content-Type', 'application/octet-stream')
+    self.set_header('Content-Disposition', 'attachment; filename=' + file_name)
+
+    buffer_size = 64 * 1024
+    with open(download_path, 'rb') as fp:
+      content = fp.read()
+      content_size = len(content)
+      buffer_segments = content_size // buffer_size
+      for buffer_seg_i in range(buffer_segments):
+        buffer_data = content[buffer_seg_i * buffer_size: (buffer_seg_i + 1) * buffer_size]
+        yield self.write(buffer_data)
+
+      yield self.write(content[buffer_segments * buffer_size:])
+
+    self.finish()
+
+
 class ClientResponseHandler(BaseHandler):
   @gen.coroutine
   def post(self):
@@ -512,12 +591,12 @@ class ClientResponseHandler(BaseHandler):
       return
     
     response_data = json.loads(response_data)
-    id = response_data['id']
-    if id not in BaseHandler.waiting_queue:
-      return
-    
-    # 
-    BaseHandler.waiting_queue[id].put(response_data)
+
+    # 添加进入结果池
+    self.response_cond.acquire()
+    self.response_dict[response_data['id']] = response_data
+    self.response_cond.notify_all()
+    self.response_cond.release()
 
 
 class ClientFileUploadAndProcessHandler(BaseHandler):
@@ -547,6 +626,7 @@ class ClientFileUploadAndProcessHandler(BaseHandler):
     file_paths = []
     file_names = []
     for meta in file_metas:
+      # 对上传的文件创建唯一标识名字
       _file_name = '%s_%s'%(str(uuid.uuid4()), meta['filename'])
       _file_path = os.path.join(upload_path, _file_name)
 
@@ -588,14 +668,11 @@ class ClientFileUploadAndProcessHandler(BaseHandler):
     request_param = {'id': model_data_names[0]}
     model_input = (model_datas[0], model_data_types[0], request_param)
 
-    # 设置等待队列
-    BaseHandler.waiting_queue[model_data_names[0]] = queue.Queue()
-
     # 发送到处理队列
     self.dataset_queue.put(model_input)
     
     # 异步等待响应(设置等待队列，并在异步线程中等待)
-    data_response = yield self.waitingResponse(model_data_names[0])
+    data_response = yield self.waitingResponse(model_data_names[0], time.time())
 
     # 绑定输入数据,返回
     data_response['INPUT_TYPE'] = model_data_types[0]
@@ -622,6 +699,23 @@ class PrefixRedirectHandler(BaseHandler):
     self.redirect('http://127.0.0.1:%d/%s'%(self.port, path), permanent=False)
 
 
+class PingApiHandler(BaseHandler):
+  @gen.coroutine
+  def get(self, *args, **kwargs):
+    self.set_status(status_code)
+
+    if status is None:
+      status = 'OK'
+      if status_code not in [200, 201, 204]:
+        status = 'ERROR'
+
+    self.write(json.dumps({
+      'status': status,
+      'message': message,
+      'content': content
+    }))
+
+
 class GracefulExitException(Exception):
   @staticmethod
   def sigterm_handler(signum, frame):
@@ -631,7 +725,6 @@ class GracefulExitException(Exception):
 def demo_server_start(demo_name,
                       demo_type,
                       demo_config,
-                      parent_id,
                       dataset_queue,
                       request_waiting_time=30):
   # register sig
@@ -660,7 +753,7 @@ def demo_server_start(demo_name,
       os.makedirs(demo_tempate_dir)
 
     html_template = demo_config['html_template']
-    if html_template is None:
+    if html_template is None or html_template == '':
       html_template = 'demo.html'
   
       if not os.path.exists(os.path.join(demo_tempate_dir, html_template)):
@@ -670,6 +763,7 @@ def demo_server_start(demo_name,
       shutil.copy(html_template, demo_tempate_dir)
 
     tornado.options.parse_command_line()
+    response_dict = {}
     settings = {
       'template_path': demo_tempate_dir,
       'static_path': demo_static_dir,
@@ -684,17 +778,21 @@ def demo_server_start(demo_name,
       'support_user_interaction': demo_config['interaction']['support_user_interaction'],
       'support_user_constraint': demo_config['interaction']['support_user_constraint'],
       'dataset_queue': dataset_queue,
-      'request_waiting_time': request_waiting_time
+      'request_waiting_time': request_waiting_time,
+      'response_dict': response_dict,
+      'response_cond': threading.Condition()
     }
-    app = tornado.web.Application(handlers=[(r"/", IndexHandler),
-                                            (r"/demo", IndexHandler),
-                                            (r"/api/query/", ClientQueryHandler),
-                                            (r"/api/comment/", ClientCommentHandler),
-                                            (r"/api/response/", ClientResponseHandler),
-                                            (r"/api/cli-query/", ClientCliQueryHandler),
-                                            (r"/submit/", ClientFileUploadAndProcessHandler),
-                                            (r"/.*/static/.*", PrefixRedirectHandler)],
-      **settings)
+    app = tornado.web.Application(handlers=[
+      (r"/", IndexHandler),
+      (r"/antgo/api/demo/query/", ClientQueryHandler),
+      (r"/antgo/api/demo/comment/", ClientCommentHandler),
+      (r"/antgo/api/demo/response/", ClientResponseHandler),
+      (r"/antgo/api/demo/cli_query/", ClientCliQueryHandler),
+      (r"/antgo/api/demo/download/", ClientDownloadHandler),
+      (r"/antgo/api/demo/submit/", ClientFileUploadAndProcessHandler),
+      (r"/antgo/api/ping/", PingApiHandler),
+      (r"/.*/static/.*", PrefixRedirectHandler)
+    ], **settings)
     http_server = tornado.httpserver.HTTPServer(app)
     http_server.listen(options.port)
   
@@ -704,6 +802,4 @@ def demo_server_start(demo_name,
   except GracefulExitException:
     logger.info('demo server exit')
     sys.exit(0)
-  except KeyboardInterrupt:
-    os.kill(parent_id, signal.SIGKILL)
-    
+
