@@ -10,6 +10,14 @@ from .interhand26M import InterHand26M
 from .lsp import LSP
 from .visalso import VisalSO
 from .imagenet import ImageNet
+from .vggface import VGGFace
+from .mnist import Mnist
+from .flic import FLIC
+from .lip import LIP
+from .ade20k import ADE20K
+from .lfw import LFW
+from .cityscape import Cityscape
+
 __all__ = [
-  'Cifar10', 'Cifar100', 'ImageNet', 'Pascal2007', 'Pascal2012', 'COCO2017', 'LSP', 'VisalSO', 'Dataset'
+  'Mnist', 'Cifar10', 'Cifar100', 'ImageNet', 'Pascal2007', 'Pascal2012', 'COCO2017', 'InterHand26M', 'LSP', 'VisalSO', 'VGGFace', 'FLIC', 'LIP', 'ADE20K', 'LFW', 'Cityscape','Dataset'
 ]
