@@ -20,6 +20,7 @@ from antgo.dataflow.basic import *
 import requests
 from antgo.context import *
 from antgo.utils import *
+import cv2
 
 
 class RecorderNode(Node):
@@ -158,9 +159,11 @@ class RecorderNode2(object):
     gts = []
     if type(val) == list or type(val) == tuple:
       for a in val:
+        assert('PREDICT' in a and 'GT' in a)
         predicts.append(a['PREDICT']['data'])
         gts.append(a['GT']['data'])
     else:
+      assert('PREDICT' in val and 'GT' in val)
       predicts = [val['PREDICT']['data']]
       gts = [val['GT']['data']]
 
@@ -506,6 +509,8 @@ class LocalRecorderNodeV2(object):
       file_name = '%d_%d_%s.png'%(count, index, name)
       # scipy.misc.imsave(os.path.join(self.dump_dir, file_name), transfer_result)
       # imageio.imwrite(os.path.join(self.dump_dir, file_name), transfer_result)
+      if not os.path.exists(self.dump_dir):
+        os.makedirs(self.dump_dir)
       cv2.imwrite(os.path.join(self.dump_dir, file_name), transfer_result)
       return file_name
     elif data_type == 'VIDEO':
@@ -555,6 +560,15 @@ class LocalRecorderNodeV2(object):
       # 重新组织数据格式
       data = {}
       group = []
+      assert('id' in result or 'image_file' in result)
+      id = ''
+      if 'id' in result:
+        id = result['id']
+        result.pop('id')
+      if 'image_file' in result:
+        id = result['image_file']
+        result.pop('image_file')
+
       for key, value in result.items():
         data_name = key
         if data_name not in data:
@@ -581,9 +595,20 @@ class LocalRecorderNodeV2(object):
             data[data_name]['data'] = value['DATA']
             value.pop('DATA')
 
+        # if 'id' in value or 'ID' in value:
+        #   if 'id' in value:
+        #     data[data_name]['id'] = value['id']
+        #     value.pop('id')
+        #   else:
+        #     data[data_name]['id'] = value['ID']
+        #     value.pop('ID')
+
         # 将value中的其它作为params
         data[data_name]['params'] = value
 
+      for key, value in data.items():
+        data[key]['id'] = id
+
       # 转换数据格式（记录到文件）
       for data_name, data_content in data.items():
         if data[data_name]['type'] not in ['FILE', 'JSON', 'SCALAR', 'STRING', 'IMAGE', 'VIDEO', 'AUDIO']:
@@ -604,14 +629,9 @@ class LocalRecorderNodeV2(object):
         self._callback([group])
       
       # 针对保留字段，进行特殊处理
-      GT = {}
-      if 'PREDICT' in data:
-        if self._record_writer is None:
-          self._record_writer = RecordWriter(self._dump_dir)
-        
-        if 'GT' in data:
-          GT = data['GT']['data']          
-        self._record_writer.write(Sample(groundtruth=GT, predict=data['PREDICT']['data']))
+      if self._record_writer is not None:
+        if 'PREDICT' in data and 'GT' in data:
+          self._record_writer.write(Sample(groundtruth=data['GT']['data'], predict=data['PREDICT']['data']))
 
     self._count += 1
 
